package cn.net.caoyao.cpzshandset.db;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class SheepExDBHelper extends SQLiteOpenHelper {
	private static final int VERSION = 1;// 提交状态增加一个生产功能
	public final static String db_name = "sheepExHandset.db";
	/**
	 * 产品信息表
	 */
	public static final String tb_product_info = "productInfo";

	/**
	 * 单标操作表
	 */
	public static final String tb_single_label_option = "singleLabelOption";
	/**
	 * 批量标签操作表
	 */
	public static final String tb_batch_label_option = "batchLabelOption";
	/**
	 * 上传记录
	 */
	public static final String tb_upload_status = "uploadStatus";
	/**
	 * 经销商列表
	 */
	public static final String tb_sheep_dealer_info="dealerInfo";
	/**
	 * 调货操作表
	 */
	public static final String tb_sheep_exchange_cargo_option="sheepExchangeCargo";


	public SheepExDBHelper(Context context) {
		super(context, db_name, null, VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		createProductInfo(db);
		createSingleLabelOption(db);
		createBatchLabelOption(db);
		createUploadStatus(db);
		createDealerInfo(db);
		createExchangeCargoOption(db);
	}
	/**
	 * id：主键自增
	 * product_id：产品id
	 * from_dealer_id：发货方经销商id
	 * to_dealer_id：收货方经销商id
	 * exchange_count：调货数量
	 * option_mode：操作方式 0：单标激活 1：批量激活
	 * status：提交状态 0：未提交 1：提交失败
	 * error_code：服务器返回的错误码
	 * createDate：创建时间
	 * @param db
	 */
	private void createExchangeCargoOption(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_sheep_exchange_cargo_option + " (id INTEGER PRIMARY KEY AUTOINCREMENT,product_id INTEGER,from_dealer_id INTEGER,to_dealer_id INTEGER,to_dealer_sale_area INTEGER,exchange_count INTEGER,option_mode INTEGER default 0,status INTEGER default 0,error_code INTEGER,createDate DATE DEFAULT (date('now','localtime')),updateDate DATE DEFAULT (date('now','localtime')))";
		db.execSQL(sql);
	}

	/**
	 * id:主键，不重复，从服务器获取
	 * province：经销商所在省
	 * market：经销商所在市
	 * name：经销商名称
	 * phone：经销商电话
	 * code：经销商编码
	 * remark：备注信息
	 * createTime：创建时间
	 * area_code：经销商所在区域的code码
	 * status：状态
	 * @param db
	 */
	private void createDealerInfo(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_sheep_dealer_info +" (id INTEGER PRIMARY KEY,province VARCHAR(20),market VARCHAR(20),name VARCHAR(20),phone VARCHAR(20),code VARCHAR(20),remark VARCHAR(20),createTime VARCHAR(20),area_code INTEGER,status INTEGER)";
		db.execSQL(sql);
	}

	/**
	 * id：表ID,主键自增
	 * option_type：操作类型（其中的插入值对应于HansetOperationConstant类中的值）
	 * label_count：上传的标签数量
	 * status：提交状态  0：上传成功 1：上传失败
	 * error_code：服务器返回的错误码
	 * upload_date：创建时间
	 * @param db
	 */
	private void createUploadStatus(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_upload_status + " (id INTEGER PRIMARY KEY AUTOINCREMENT,option_type INTEGER,label_count INTEGER,status INTEGER,error_code INTEGER,upload_date DATE DEFAULT (date('now','localtime')))";
		db.execSQL(sql);
	}

	




	/**
	 * start_label_id：起始标签
	 * end_label_id：结束标签
	 * option_type：操作类型（其中的插入值对应于HansetOperationConstant类中的值）
	 * option_id：关联的表的id
	 * @param db
	 */
	private void createBatchLabelOption(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_batch_label_option + " (id INTEGER PRIMARY KEY AUTOINCREMENT,start_label_id bigint,end_label_id bigint,option_type INTEGER,option_id INTEGER)";
		db.execSQL(sql);
	}

	/**
	 * label_id：扫描的标签id   
	 * option_type：操作类型（其中的插入值对应于HansetOperationConstant类中的值）
	 * option_id：关联的表的id
	 * labelStatus：当前标签的状态   0:默认值    1:不合法标签    2:重复激活标签     3:未激活标签
	 * @param db
	 */
	private void createSingleLabelOption(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_single_label_option + " (id INTEGER PRIMARY KEY AUTOINCREMENT,label_id bigint,option_type INTEGER,option_id INTEGER,labelStatus INTEGER default 0)";
		db.execSQL(sql);	
	}


	/**
	 * id:产品id，服务器返回的数据直接插入
	 * name：产品名称
	 * namePY：产品名称的拼音
	 * nid：产品编号（即，商品条码）
	 * @param db
	 */
	private void createProductInfo(SQLiteDatabase db) {
		String sql = "CREATE TABLE " + tb_product_info + " (id VARCHAR(20),name VARCHAR(20), namePY VARCHAR(20),nid VARCHAR(20))";
		db.execSQL(sql);		
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
	}
	
	/**
	 * 检测数据库中是否存在某一列
	 * @param db
	 * @param tableName
	 * @param primaryKeyName
	 * @param columnName
	 * @return true:存在    false:不存在
	 */
	private boolean checkColumnExists(SQLiteDatabase db,String tableName, String primaryKeyName,String columnName){
		boolean b = true;
		String sql = "SELECT * from " + tableName +" where "+primaryKeyName+"=0";
		Cursor cursor = db.rawQuery(sql, null);
		int columnIndex = cursor.getColumnIndex(columnName);
		cursor.close();
		if (columnIndex < 0) {
			b = false;
		}
		return b;
	}

}
